//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:27 $
//
// Category     : VST 2.x SDK Samples
// Filename     : again.cpp
// Created by   : Steinberg Media Technologies
// Description  : Stereo plugin which applies Gain [-oo, 0dB]
//
//  2006, Steinberg Media Technologies, All Rights Reserved
//-------------------------------------------------------------------------------------------------------

#include "again.h"
#include <math.h>
#include <stdio.h>
//-------------------------------------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new AGain (audioMaster);
}

//-------------------------------------------------------------------------------------------------------
AGain::AGain (audioMasterCallback audioMaster)
//***
: AudioEffectX (audioMaster, 1, 2)	// 1 program, 1 parameter only
//
{
	setNumInputs (2);		// stereo in
	setNumOutputs (2);		// stereo out
	setUniqueID ('5b33');	// identify
	canProcessReplacing ();	// supports replacing output
//	canDoubleReplacing ();	// supports double precision processing
//***
	fBits = 1.f;
	fFreq = 1.f;
	fPhase=0.f;
	fLastVal[0]=fLastVal[1]=0.f;
//
	vst_strncpy (programName, "Default", kVstMaxProgNameLen);	// default program name
}

//-------------------------------------------------------------------------------------------------------
AGain::~AGain ()
{
	// nothing to do here
}

//-------------------------------------------------------------------------------------------------------
void AGain::setProgramName (char* name)
{
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void AGain::getProgramName (char* name)
{
	vst_strncpy (name, programName, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void AGain::setParameter (VstInt32 index, float value)
{
//***
	switch(index) {
	case 0:
		fBits = value;
		break;
	case 1:
		fFreq=value;
		break;
	}
}

//-----------------------------------------------------------------------------------------
float AGain::getParameter (VstInt32 index)
{
//***
	switch(index) {
	case 0:
		return fBits;
	case 1:
		return fFreq;
	}
}

//-----------------------------------------------------------------------------------------
void AGain::getParameterName (VstInt32 index, char* label)
{
//***
	switch(index) {
	case 0:
		vst_strncpy (label, "Bits", kVstMaxParamStrLen);
		break;
	case 1:
		vst_strncpy(label,"Freq",kVstMaxParamStrLen);
		break;
	}
}

//-----------------------------------------------------------------------------------------
void AGain::getParameterDisplay (VstInt32 index, char* text)
{
//***
	float fStep;
	switch(index) {
	case 0:
		fStep=fBits*254.f+2.f;
		if(fBits==1.0)
			strcpy(text,"--");
		else
			sprintf(text,"%f",log10(fStep)/log10(2.0));
		break;
	case 1:
		if(fFreq==1.0)
			strcpy(text,"--");
		else
			sprintf(text,"%f",fFreq*47.+1.);
		break;
	}
//	dB2string (fGain, text, kVstMaxParamStrLen);
}

//-----------------------------------------------------------------------------------------
void AGain::getParameterLabel (VstInt32 index, char* label)
{
//***
	switch(index) {
	case 0:
		vst_strncpy (label, "bits", kVstMaxParamStrLen);
		break;
	case 1:
		vst_strncpy (label, "kHz",kVstMaxParamStrLen);
		break;
	}
}

//------------------------------------------------------------------------
bool AGain::getEffectName (char* name)
{
	vst_strncpy (name, "BitCrusher", kVstMaxEffectNameLen);
	return true;
}

//------------------------------------------------------------------------
bool AGain::getProductString (char* text)
{
	vst_strncpy (text, "BitCrusher", kVstMaxProductStrLen);
	return true;
}

//------------------------------------------------------------------------
bool AGain::getVendorString (char* text)
{
	vst_strncpy (text, "Test", kVstMaxVendorStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
VstInt32 AGain::getVendorVersion ()
{ 
	return 1000; 
}

float AGain::DoEffect(int iCh,float x) {
	float fStep;
	float fTemp;
	fPhase-=1.0/sampleRate;
	if(fFreq==1.0 || fPhase<0.f) {
		fPhase+=1.0/(fFreq*47000.+1000);
		if(fBits<1.f) {
			fStep=fBits*254.f+2.f;
			x=(float)((int)(x*fStep))/fStep;
		}
		fLastVal[iCh]=x;
	}
	return fLastVal[iCh];
}
//-----------------------------------------------------------------------------------------
void AGain::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
    float* in1  =  inputs[0];
    float* in2  =  inputs[1];
    float* out1 = outputs[0];
    float* out2 = outputs[1];

    while (--sampleFrames >= 0)
    {
//***
		(*out1++) = DoEffect(0,*in1++);
		(*out2++) = DoEffect(1,*in2++);
    }
}

//-----------------------------------------------------------------------------------------
void AGain::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
    double* in1  =  inputs[0];
    double* in2  =  inputs[1];
    double* out1 = outputs[0];
    double* out2 = outputs[1];
	double dGain = fBits;

    while (--sampleFrames >= 0)
    {
        (*out1++) = (*in1++) * dGain;
        (*out2++) = (*in2++) * dGain;
    }
}
